@@ -11,6 +11,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+function container_runtime {
+    if [[ ${CONTAINER_ENGINE} == 'crio' ]]; then
+        sudo podman "$@"
+    else
+        docker "$@"
+    fi
+}
+
 function create_kuryr_account {
     if is_service_enabled kuryr-kubernetes; then
         create_service_user "kuryr" "admin"
@@ -32,19 +40,6 @@ function create_kuryr_lock_dir {
     sudo install -d -o "$STACK_USER" "$KURYR_LOCK_DIR"
 }
 
-function get_distutils_data_path {
-    cat << EOF | python -
-from __future__ import print_function
-import distutils.dist
-import distutils.command.install
-
-inst = distutils.command.install.install(distutils.dist.Distribution())
-inst.finalize_options()
-
-print(inst.install_data)
-EOF
-}
-
 function configure_kuryr {
     local dir
     sudo install -d -o "$STACK_USER" "$KURYR_CONFIG_DIR"
@@ -138,7 +133,7 @@ function configure_kuryr {
 function generate_containerized_kuryr_resources {
     local cni_daemon
     cni_daemon=$1
-    if [[ KURYR_CONTROLLER_REPLICAS -eq 1 ]]; then
+    if [[ $KURYR_CONTROLLER_REPLICAS -eq 1 ]]; then
         KURYR_CONTROLLER_HA="False"
     else
         KURYR_CONTROLLER_HA="True"
@@ -211,7 +206,7 @@ function copy_tempest_kubeconfig {
     tempest_home='/home/tempest'
     if is_service_enabled openshift-master; then
         sudo mkdir -p "${HOME}/.kube"
-        sudo cp "${OPENSHIFT_DATA_DIR}/admin.kubeconfig" "${HOME}/.kube/config"
+        sudo cp "${OPENSHIFT_DATA_DIR}/master/admin.kubeconfig" "${HOME}/.kube/config"
         sudo chown -R $STACK_USER "${HOME}/.kube"
     fi
 
@@ -264,15 +259,10 @@ function create_k8s_api_service {
         address="${HOST_IP}"
     fi
 
-    use_octavia=$(trueorfalse True KURYR_K8S_LBAAS_USE_OCTAVIA)
-    if [[ "$use_octavia" == "True" && \
-          "$KURYR_K8S_OCTAVIA_MEMBER_MODE" == "L2" ]]; then
-        create_load_balancer_member "$(hostname)" "$address" "$api_port" \
-            default/kubernetes:${KURYR_K8S_API_LB_PORT} $KURYR_NEUTRON_DEFAULT_POD_SUBNET "$lb_name" "$project_id"
-    else
-        create_load_balancer_member "$(hostname)" "$address" "$api_port" \
-            default/kubernetes:${KURYR_K8S_API_LB_PORT} public-subnet "$lb_name" "$project_id"
-    fi
+    # Regardless of the octavia mode, the k8s API will be behind an L3 mode
+    # amphora driver loadbalancer
+    create_load_balancer_member "$(hostname)" "$address" "$api_port" \
+        default/kubernetes:${KURYR_K8S_API_LB_PORT} public-subnet "$lb_name" "$project_id"
 }
 
 function configure_neutron_defaults {
@@ -285,6 +275,7 @@ function configure_neutron_defaults {
     local router_id
     local ext_svc_net_id
     local ext_svc_subnet_id
+    local prot
 
     # If a subnetpool is not passed, we get the one created in devstack's
     # Neutron module
@@ -325,24 +316,35 @@ function configure_neutron_defaults {
     ext_svc_subnet_id="$(openstack subnet show -c id -f value \
         "${KURYR_NEUTRON_DEFAULT_EXT_SVC_SUBNET}")"
 
+    # In order for the ports to allow service traffic under Octavia L3 mode,
+    # it is necessary for the service subnet to be allowed into the port's
+    # security groups. If L3 is used, then the pods created will include it.
+    # Otherwise it will be just used by the kubelet port used for the K8s API
+    # load balancer
+    local service_cidr
+    local service_pod_access_sg_id
+    service_cidr=$(openstack --os-cloud devstack-admin \
+        --os-region "$REGION_NAME" subnet show \
+        "${KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET}" -f value -c cidr)
+    service_pod_access_sg_id=$(openstack --os-cloud devstack-admin \
+        --os-region "$REGION_NAME" \
+        security group create --project "$project_id" \
+        service_pod_access -f value -c id)
+    openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
+        security group rule create --project "$project_id" \
+        --description "k8s service subnet allowed" \
+        --remote-ip "$service_cidr" --ethertype IPv4 --protocol tcp \
+        "$service_pod_access_sg_id"
+    # Since Octavia supports also UDP load balancing, we need to allow
+    # also udp traffic
+    openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
+        security group rule create --project "$project_id" \
+        --description "k8s service subnet UDP allowed" \
+        --remote-ip "$service_cidr" --ethertype IPv4 --protocol udp \
+        "$service_pod_access_sg_id"
+
     if [[ "$use_octavia" == "True" && \
           "$KURYR_K8S_OCTAVIA_MEMBER_MODE" == "L3" ]]; then
-        # In order for the pods to allow service traffic under Octavia L3 mode,
-        #it is necessary for the service subnet to be allowed into the $sg_ids
-        local service_cidr
-        local service_pod_access_sg_id
-        service_cidr=$(openstack --os-cloud devstack-admin \
-            --os-region "$REGION_NAME" subnet show \
-            "${KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET}" -f value -c cidr)
-        service_pod_access_sg_id=$(openstack --os-cloud devstack-admin \
-            --os-region "$REGION_NAME" \
-            security group create --project "$project_id" \
-            service_pod_access -f value -c id)
-        openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
-            security group rule create --project "$project_id" \
-            --description "k8s service subnet allowed" \
-            --remote-ip "$service_cidr" --ethertype IPv4 --protocol tcp \
-            "$service_pod_access_sg_id"
         if [ -n "$sg_ids" ]; then
             sg_ids+=",${service_pod_access_sg_id}"
         else
@@ -370,6 +372,13 @@ function configure_neutron_defaults {
             --description "k8s pod subnet allowed from k8s-pod-subnet" \
             --remote-ip "$pod_cidr" --ethertype IPv4 --protocol tcp \
             "$octavia_pod_access_sg_id"
+        # Since Octavia supports also UDP load balancing, we need to allow
+        # also udp traffic
+        openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
+            security group rule create --project "$project_id" \
+            --description "k8s pod subnet allowed from k8s-pod-subnet" \
+            --remote-ip "$pod_cidr" --ethertype IPv4 --protocol udp \
+            "$octavia_pod_access_sg_id"
         if [ -n "$sg_ids" ]; then
             sg_ids+=",${octavia_pod_access_sg_id}"
         else
@@ -377,22 +386,6 @@ function configure_neutron_defaults {
         fi
     fi
 
-    KURYR_K8S_CONTAINERIZED_DEPLOYMENT=$(trueorfalse False KURYR_K8S_CONTAINERIZED_DEPLOYMENT)
-    if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "False" ]; then
-        local service_cidr
-        local k8s_api_clusterip
-        service_cidr=$(openstack --os-cloud devstack-admin \
-                                 --os-region "$REGION_NAME" \
-                                 subnet show "$KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET" \
-                                 -c cidr -f value)
-        k8s_api_clusterip=$(_cidr_range "$service_cidr" | cut -f1)
-        # NOTE(dulek): KURYR_K8S_API_LB_URL will be a global to be used by next
-        #              deployment phases.
-        KURYR_K8S_API_LB_URL="https://${k8s_api_clusterip}:${KURYR_K8S_API_LB_PORT}"
-        iniset "$KURYR_CONFIG" kubernetes api_root ${KURYR_K8S_API_LB_URL}
-    else
-        iniset "$KURYR_CONFIG" kubernetes api_root '""'
-    fi
     iniset "$KURYR_CONFIG" neutron_defaults project "$project_id"
     iniset "$KURYR_CONFIG" neutron_defaults pod_subnet "$pod_subnet_id"
     iniset "$KURYR_CONFIG" neutron_defaults pod_security_groups "$sg_ids"
@@ -412,26 +405,38 @@ function configure_neutron_defaults {
             --os-region "$REGION_NAME" \
             security group create --project "$project_id" \
             allow_from_default -f value -c id)
+
+        for prot in icmp tcp udp ;
+          do
+            openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
+              security group rule create --project "$project_id" \
+              --description "allow traffic from default namespace" \
+              --remote-group "$allow_namespace_sg_id" --ethertype IPv4 --protocol "$prot" \
+              "$allow_default_sg_id"
+
+            if [ "$prot" != "icmp" ] ; then
+              openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
+              security group rule create --project "$project_id" \
+              --description "allow traffic from namespaces at default namespace" \
+              --remote-group "$allow_default_sg_id" --ethertype IPv4 --protocol "$prot" \
+              "$allow_namespace_sg_id"
+            fi
+          done
+
+        # NOTE(ltomasbo): Some tempest test are using FIP and depends on icmp
+        # traffic being allowed to the pods. To enable these tests we permit
+        # icmp traffic from everywhere on the default namespace. Note tcp
+        # traffic will be dropped, just icmp is permitted.
         openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
             security group rule create --project "$project_id" \
-            --description "allow traffic from default namespace" \
-            --remote-group "$allow_namespace_sg_id" --ethertype IPv4 --protocol tcp \
-            "$allow_default_sg_id"
-        openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
-            security group rule create --project "$project_id" \
-            --description "allow traffic from default namespace" \
-            --remote-group "$allow_namespace_sg_id" --ethertype IPv4 --protocol icmp \
-            "$allow_default_sg_id"
-        openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
-            security group rule create --project "$project_id" \
-            --description "allow traffic from namespaces at default namespace" \
-            --remote-group "$allow_default_sg_id" --ethertype IPv4 --protocol tcp \
-            "$allow_namespace_sg_id"
-        openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
-            security group rule create --project "$project_id" \
-            --description "allow traffic from namespaces at default namespace" \
-            --remote-group "$allow_default_sg_id" --ethertype IPv4 --protocol icmp \
-            "$allow_namespace_sg_id"
+            --description "allow imcp traffic from everywhere to default namespace" \
+            --ethertype IPv4 --protocol icmp "$allow_namespace_sg_id"
+
+        # NOTE(ltomasbo): As more security groups and rules are created, there
+        # is a need to increase the quota for it
+         openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
+             quota set --secgroups 100 --secgroup-rules 100 "$project_id"
+
 
         iniset "$KURYR_CONFIG" namespace_sg sg_allow_from_namespaces "$allow_namespace_sg_id"
         iniset "$KURYR_CONFIG" namespace_sg sg_allow_from_default "$allow_default_sg_id"
@@ -441,6 +446,7 @@ function configure_neutron_defaults {
     fi
     iniset "$KURYR_CONFIG" neutron_defaults external_svc_net "$ext_svc_net_id"
     iniset "$KURYR_CONFIG" octavia_defaults member_mode "$KURYR_K8S_OCTAVIA_MEMBER_MODE"
+    iniset "$KURYR_CONFIG" octavia_defaults sg_mode "$KURYR_K8S_OCTAVIA_SG_MODE"
     if [[ "$use_octavia" == "True" ]]; then
         # Octavia takes a very long time to start the LB in the gate. We need
         # to tweak the timeout for the LB creation. Let's be generous and give
@@ -448,6 +454,7 @@ function configure_neutron_defaults {
         # FIXME(dulek): This might be removed when bug 1753653 is fixed and
         #               Kuryr restarts waiting for LB on timeouts.
         iniset "$KURYR_CONFIG" neutron_defaults lbaas_activation_timeout 1200
+        iniset "$KURYR_CONFIG" kubernetes endpoints_driver_octavia_provider "$KURYR_EP_DRIVER_OCTAVIA_PROVIDER"
     fi
 }
 
@@ -465,13 +472,6 @@ function configure_k8s_pod_sg_rules {
     create_k8s_icmp_sg_rules "$sg_id" ingress
 }
 
-function get_hyperkube_container_cacert_setup_dir {
-    case "$1" in
-        1.[0-3].*) echo "/data";;
-        *) echo "/srv/kubernetes"
-    esac
-}
-
 function create_token() {
   echo $(cat /dev/urandom | base64 | tr -d "=+/" | dd bs=32 count=1 2> /dev/null)
 }
@@ -542,12 +542,43 @@ function wait_for {
     echo ""
 }
 
+function wait_for_ok_health {
+    local name
+    local url
+    local cacert_path
+    local start_time
+    local key_path
+    local cert_path
+    local curr_time
+    local time_diff
+    name="$1"
+    url="$2"
+    cacert_path=${3:-}
+    key_path=${4:-}
+    cert_path=${5:-}
+    timeout=${6:-$KURYR_WAIT_TIMEOUT}
+
+
+    extra_flags=('-s' ${cacert_path:+--cacert "$cacert_path"} ${key_path:+--key "$key_path"} ${cert_path:+--cert "$cert_path"})
+
+    start_time=$(date +%s)
+    echo -n "Waiting for $name to be healthy"
+    until [[ "$(curl "${extra_flags[@]}" "$url")" == "ok" ]]; do
+        echo -n "."
+        curr_time=$(date +%s)
+        time_diff=$((curr_time - start_time))
+        [[ $time_diff -le $timeout ]] || die "Timed out waiting for $name"
+        sleep 1
+    done
+    echo ""
+}
+
 function run_k8s_api {
     local service_cidr
     local cluster_ip_range
 
     # Runs Hyperkube's Kubernetes API Server
-    wait_for "etcd" "${KURYR_ETCD_ADVERTISE_CLIENT_URL}/v2/machines"
+    wait_for "etcd" "http://${SERVICE_HOST}:${ETCD_PORT}/v2/machines"
 
     service_cidr=$(openstack --os-cloud devstack-admin \
                          --os-region "$REGION_NAME" \
@@ -559,56 +590,71 @@ function run_k8s_api {
         cluster_ip_range="$service_cidr"
     fi
 
-    run_container kubernetes-api \
-        --net host \
-        --volume="${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw" \
-        "${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}" \
-            /hyperkube apiserver \
-                --service-cluster-ip-range="${cluster_ip_range}" \
-                --insecure-bind-address=0.0.0.0 \
-                --insecure-port="${KURYR_K8S_API_PORT}" \
-                --etcd-servers="${KURYR_ETCD_ADVERTISE_CLIENT_URL}" \
-                --admission-control=NamespaceLifecycle,LimitRanger,ServiceAccount,ResourceQuota \
-                --client-ca-file=/srv/kubernetes/ca.crt \
-                --basic-auth-file=/srv/kubernetes/basic_auth.csv \
-                --min-request-timeout=300 \
-                --tls-cert-file=/srv/kubernetes/server.cert \
-                --tls-private-key-file=/srv/kubernetes/server.key \
-                --token-auth-file=/srv/kubernetes/known_tokens.csv \
-                --allow-privileged=true \
-                --v=2 \
-                --logtostderr=true
+    local command
+    command=(--net=host
+             --volume=${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw)
+    if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
+        command+=(--restart=on-failure)
+    fi
+    command+=(${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}
+              /hyperkube apiserver
+                --service-cluster-ip-range=${cluster_ip_range}
+                --insecure-bind-address=0.0.0.0
+                --insecure-port=${KURYR_K8S_API_PORT}
+                --etcd-servers=http://${SERVICE_HOST}:${ETCD_PORT}
+                --admission-control=NamespaceLifecycle,LimitRanger,ServiceAccount,ResourceQuota
+                --client-ca-file=/srv/kubernetes/ca.crt
+                --basic-auth-file=/srv/kubernetes/basic_auth.csv
+                --min-request-timeout=300
+                --tls-cert-file=/srv/kubernetes/server.cert
+                --tls-private-key-file=/srv/kubernetes/server.key
+                --token-auth-file=/srv/kubernetes/known_tokens.csv
+                --allow-privileged=true
+                --v=2
+                --logtostderr=true)
+
+    run_container kubernetes-api "${command[@]}"
 }
 
 function run_k8s_controller_manager {
     # Runs Hyperkube's Kubernetes controller manager
     wait_for "Kubernetes API Server" "$KURYR_K8S_API_URL"
 
-    run_container kubernetes-controller-manager \
-        --net host \
-        --volume="${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw" \
-        "${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}" \
-            /hyperkube controller-manager \
-                --master="$KURYR_K8S_API_URL" \
-                --service-account-private-key-file=/srv/kubernetes/server.key \
-                --root-ca-file=/srv/kubernetes/ca.crt \
-                --min-resync-period=3m \
-                --v=2 \
-                --logtostderr=true
+    local command
+    command=(--net=host
+             --volume=${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw)
+    if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
+        command+=(--restart=on-failure)
+    fi
+    command+=(${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}
+              /hyperkube controller-manager
+                --master=$KURYR_K8S_API_URL
+                --service-account-private-key-file=/srv/kubernetes/server.key
+                --root-ca-file=/srv/kubernetes/ca.crt
+                --min-resync-period=3m
+                --v=2
+                --logtostderr=true)
+
+    run_container kubernetes-controller-manager "${command[@]}"
 }
 
 function run_k8s_scheduler {
     # Runs Hyperkube's Kubernetes scheduler
     wait_for "Kubernetes API Server" "$KURYR_K8S_API_URL"
 
-    run_container kubernetes-scheduler \
-        --net host \
-        --volume="${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw" \
-        "${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}" \
-            /hyperkube scheduler \
-                --master="$KURYR_K8S_API_URL" \
-                --v=2 \
-                --logtostderr=true
+    local command
+    command=(--net=host
+             --volume=${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw)
+    if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
+        command+=(--restart=on-failure)
+    fi
+    command+=(${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}
+              /hyperkube scheduler
+                --master=$KURYR_K8S_API_URL
+                --v=2
+                --logtostderr=true)
+
+    run_container kubernetes-scheduler "${command[@]}"
 }
 
 function prepare_kubeconfig {
@@ -627,16 +673,24 @@ function extract_hyperkube {
     tmp_loopback_cni_path="/tmp/loopback"
     tmp_nsenter_path="/tmp/nsenter"
 
-    hyperkube_container=$(docker run -d \
+    hyperkube_container=$(container_runtime run -d \
         --net host \
        "${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}" \
        /bin/false)
-    docker cp "${hyperkube_container}:/hyperkube" "$tmp_hyperkube_path"
-    docker cp "${hyperkube_container}:/opt/cni/bin/loopback" \
-        "$tmp_loopback_cni_path"
-    docker cp "${hyperkube_container}:/usr/bin/nsenter" "$tmp_nsenter_path"
+    if [[ ${CONTAINER_ENGINE} == 'crio' ]]; then
+        mnt=`container_runtime mount "${hyperkube_container}"`
+        sudo cp "${mnt}/hyperkube" "$tmp_hyperkube_path"
+        sudo cp "${mnt}/opt/cni/bin/loopback" "$tmp_loopback_cni_path"
+        sudo cp "${mnt}/usr/bin/nsenter" "$tmp_nsenter_path"
+        container_runtime umount ${hyperkube_container}
+    else
+        container_runtime cp "${hyperkube_container}:/hyperkube" "$tmp_hyperkube_path"
+        container_runtime cp "${hyperkube_container}:/opt/cni/bin/loopback" \
+            "$tmp_loopback_cni_path"
+        container_runtime cp "${hyperkube_container}:/usr/bin/nsenter" "$tmp_nsenter_path"
+    fi
 
-    docker rm "$hyperkube_container"
+    container_runtime rm --force "$hyperkube_container"
     sudo install -o "$STACK_USER" -m 0555 -D "$tmp_hyperkube_path" \
         "$KURYR_HYPERKUBE_BINARY"
     sudo install -o "$STACK_USER" -m 0555 -D "$tmp_loopback_cni_path" \
@@ -668,16 +722,12 @@ function run_k8s_kubelet {
     # adding Python and all our CNI/binding dependencies.
     local command
     local minor_version
-    local cgroup_driver
-
-    cgroup_driver="$(docker info|awk '/Cgroup/ {print $NF}')"
 
     sudo mkdir -p "${KURYR_HYPERKUBE_DATA_DIR}/"{kubelet,kubelet.cert}
     command="$KURYR_HYPERKUBE_BINARY kubelet\
-        --kubeconfig=${HOME}/.kube/config --require-kubeconfig \
+        --kubeconfig=${HOME}/.kube/config \
         --allow-privileged=true \
         --v=2 \
-        --cgroup-driver=$cgroup_driver \
         --address=0.0.0.0 \
         --enable-server \
         --network-plugin=cni \
@@ -686,9 +736,31 @@ function run_k8s_kubelet {
         --cert-dir=${KURYR_HYPERKUBE_DATA_DIR}/kubelet.cert \
         --root-dir=${KURYR_HYPERKUBE_DATA_DIR}/kubelet"
 
+    if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
+        command+=" --cgroup-driver $(docker info|awk '/Cgroup/ {print $NF}')"
+    elif [[ ${CONTAINER_ENGINE} == 'crio' ]]; then
+        local crio_conf
+        crio_conf=/etc/crio/crio.conf
+
+        command+=" --cgroup-driver=$(iniget ${crio_conf} crio.runtime cgroup_manager)"
+        command+=" --container-runtime=remote --container-runtime-endpoint=unix:///var/run/crio/crio.sock --runtime-request-timeout=10m"
+
+        # We need to reconfigure CRI-O in this case as well.
+        # FIXME(dulek): This should probably go to devstack-plugin-container
+        iniset -sudo ${crio_conf} crio.network network_dir \"${CNI_CONF_DIR}\"
+        iniset -sudo ${crio_conf} crio.network plugin_dir \"${CNI_BIN_DIR}\"
+        sudo systemctl --no-block restart crio.service
+    fi
+
+    declare -r min_not_require_kubeconfig_ver="1.10.0"
+    if [[ "$KURYR_HYPERKUBE_VERSION" == "$(echo -e "${KURYR_HYPERKUBE_VERSION}\n${min_not_require_kubeconfig_ver}" | sort -V | head -n 1)" ]]; then
+        # Version 1.10 did away with that config option
+        command+=" --require-kubeconfig"
+    fi
+
     # Kubernetes 1.8+ requires additional option to work in the gate.
-    minor_version=${KURYR_HYPERKUBE_VERSION:3:1}
-    if [ ${minor_version} -gt 7 ]; then
+    declare -r min_no_swap_ver="1.8.0"
+    if [[ "$min_no_swap_ver" == "$(echo -e "${KURYR_HYPERKUBE_VERSION}\n${min_no_swap_ver}" | sort -V | head -n 1)" ]]; then
         command="$command --fail-swap-on=false"
     fi
 
@@ -709,15 +781,17 @@ function run_kuryr_kubernetes {
     local python_bin=$(which python)
     if is_service_enabled openshift-master; then
         wait_for "OpenShift API Server" "$KURYR_K8S_API_LB_URL" \
-            "${OPENSHIFT_DATA_DIR}/ca.crt" 1200
+            "${OPENSHIFT_DATA_DIR}/master/ca.crt" 1200
     else
-        wait_for "Kubernetes API Server" "$KURYR_K8S_API_LB_URL" \
-            "${KURYR_HYPERKUBE_DATA_DIR}/kuryr-ca.crt" 1200
+        wait_for_ok_health "Kubernetes API Server" "${KURYR_K8S_API_LB_URL}/healthz" \
+            "${KURYR_HYPERKUBE_DATA_DIR}/kuryr-ca.crt" \
+            "${KURYR_HYPERKUBE_DATA_DIR}/kuryr.key" \
+            "${KURYR_HYPERKUBE_DATA_DIR}/kuryr.crt" \
+            1200
     fi
 
-    run_process kuryr-kubernetes \
-        "$python_bin ${KURYR_HOME}/scripts/run_server.py  \
-            --config-file $KURYR_CONFIG"
+    local controller_bin=$(which kuryr-k8s-controller)
+    run_process kuryr-kubernetes "$controller_bin --config-file $KURYR_CONFIG"
 }
 
 
@@ -781,6 +855,56 @@ function create_ingress_l7_router {
     fi
 }
 
+function configure_overcloud_vm_k8s_svc_sg {
+    local project_id
+    local security_group
+
+    project_id=$(get_or_create_project \
+        "$KURYR_NEUTRON_DEFAULT_PROJECT" default)
+    security_group=$(openstack security group list \
+        --project "$project_id" -c ID -c Name -f value | \
+        awk '{if ($2=="default") print $1}')
+    openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
+        security group rule create --project "$project_id" \
+        --dst-port "$KURYR_K8S_API_LB_PORT" "$security_group"
+    openstack port set "$KURYR_OVERCLOUD_VM_PORT" --security-group service_pod_access
+}
+
+function update_tempest_conf_file {
+
+    local use_octavia="$1"
+
+    if [[ "$KURYR_USE_PORT_POOLS" == "True" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes port_pool_enabled True
+    fi
+    if [[ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes containerized True
+    fi
+    if [[ "$KURYR_SG_DRIVER" == "namespace" ]] && [[ "$KURYR_SUBNET_DRIVER" == "namespace" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes namespace_enabled True
+    fi
+    if [[ "$KURYR_K8S_SERIAL_TESTS" == "True" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes run_tests_serial True
+    fi
+    if [[ "$KURYR_MULTI_VIF_DRIVER" == "npwg_multiple_interfaces" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes npwg_multi_vif_enabled True
+    fi
+    if [[ "$KURYR_ENABLED_HANDLERS" =~ .*policy.* ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes network_policy_enabled True
+    fi
+    if ! is_service_enabled kuryr-daemon; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes kuryr_daemon_enabled False
+    fi
+    # NOTE(yboaron): Services with protocol UDP are supported in Kuryr
+    # starting from Stein release and only for Octavia
+    if [[ "$use_octavia" == "True" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes test_udp_services True
+    fi
+    if [[ "$KURYR_CONTROLLER_HA" == "True" ]]; then
+        iniset $TEMPEST_CONFIG kuryr_kubernetes ap_ha True
+    fi
+}
+
 source $DEST/kuryr-kubernetes/devstack/lib/kuryr_kubernetes
 
 # main loop
@@ -799,12 +923,28 @@ elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
 fi
 
 if [[ "$1" == "stack" && "$2" == "extra" ]]; then
-    if is_service_enabled kuryr-kubernetes; then
+    if [ "$KURYR_CONFIGURE_NEUTRON_DEFAULTS" == "True" ]; then
         KURYR_CONFIGURE_NEUTRON_DEFAULTS=$(trueorfalse True KURYR_CONFIGURE_NEUTRON_DEFAULTS)
-        if [ "$KURYR_CONFIGURE_NEUTRON_DEFAULTS" == "True" ]; then
+        if is_service_enabled kuryr-kubernetes; then
             configure_neutron_defaults
         fi
+
+        KURYR_K8S_CONTAINERIZED_DEPLOYMENT=$(trueorfalse False KURYR_K8S_CONTAINERIZED_DEPLOYMENT)
+        if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "False" ]; then
+            service_cidr=$(openstack --os-cloud devstack-admin \
+                                     --os-region "$REGION_NAME" \
+                                     subnet show "$KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET" \
+                                     -c cidr -f value)
+            k8s_api_clusterip=$(_cidr_range "$service_cidr" | cut -f1)
+            # NOTE(dulek): KURYR_K8S_API_LB_URL will be a global to be used by next
+            #              deployment phases.
+            KURYR_K8S_API_LB_URL="https://${k8s_api_clusterip}:${KURYR_K8S_API_LB_PORT}"
+            iniset "$KURYR_CONFIG" kubernetes api_root ${KURYR_K8S_API_LB_URL}
+        else
+            iniset "$KURYR_CONFIG" kubernetes api_root '""'
+        fi
     fi
+
     # FIXME(limao): When Kuryr start up, it need to detect if neutron
     # support tag plugin.
     #
@@ -822,11 +962,6 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
     # sure Kuryr can start before neutron-server, so Kuryr start in "extra"
     # phase.  Bug: https://bugs.launchpad.net/kuryr/+bug/1587522
 
-    if is_service_enabled legacy_etcd; then
-        prepare_etcd_legacy
-        run_etcd_legacy
-    fi
-
     # FIXME(apuimedo): Allow running only openshift node for multinode devstack
     # We are missing generating a node config so that it does not need to
     # bootstrap from the master config.
@@ -882,6 +1017,8 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
         KURYR_CONFIGURE_BAREMETAL_KUBELET_IFACE=$(trueorfalse True KURYR_CONFIGURE_BAREMETAL_KUBELET_IFACE)
         if [[ "$KURYR_CONFIGURE_BAREMETAL_KUBELET_IFACE" == "True" ]]; then
             ovs_bind_for_kubelet "$KURYR_NEUTRON_DEFAULT_PROJECT" 6443
+        else
+            configure_overcloud_vm_k8s_svc_sg
         fi
     fi
 
@@ -891,14 +1028,24 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
     fi
 
     KURYR_K8S_CONTAINERIZED_DEPLOYMENT=$(trueorfalse False KURYR_K8S_CONTAINERIZED_DEPLOYMENT)
-    if is_service_enabled kuryr-kubernetes; then
-        /usr/local/bin/kubectl apply -f ${KURYR_HOME}/kubernetes_crds/kuryrnet.yaml
+    KURYR_FORCE_IMAGE_BUILD=$(trueorfalse False KURYR_FORCE_IMAGE_BUILD)
+    if is_service_enabled kuryr-kubernetes || [[ ${KURYR_FORCE_IMAGE_BUILD} == "True" ]]; then
         if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
             if is_service_enabled kuryr-daemon; then
                 build_kuryr_containers $CNI_BIN_DIR $CNI_CONF_DIR True
-                generate_containerized_kuryr_resources True
             else
                 build_kuryr_containers $CNI_BIN_DIR $CNI_CONF_DIR False
+            fi
+        fi
+    fi
+
+    if is_service_enabled kuryr-kubernetes; then
+        /usr/local/bin/kubectl apply -f ${KURYR_HOME}/kubernetes_crds/kuryrnet.yaml
+        /usr/local/bin/kubectl apply -f ${KURYR_HOME}/kubernetes_crds/kuryrnetpolicy.yaml
+        if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
+            if is_service_enabled kuryr-daemon; then
+                generate_containerized_kuryr_resources True
+            else
                 generate_containerized_kuryr_resources False
             fi
         fi
@@ -941,20 +1088,8 @@ elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
             configure_and_run_registry
         fi
     fi
-    if is_service_enabled tempest && [[ "$KURYR_USE_PORT_POOLS" == "True" ]]; then
-        iniset $TEMPEST_CONFIG kuryr_kubernetes port_pool_enabled True
-    fi
-    if is_service_enabled tempest && [[ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]]; then
-        iniset $TEMPEST_CONFIG kuryr_kubernetes containerized True
-    fi
-    if is_service_enabled tempest && [[ "$KURYR_SG_DRIVER" == "namespace" ]] && [[ "$KURYR_SUBNET_DRIVER" == "namespace" ]]; then
-        iniset $TEMPEST_CONFIG kuryr_kubernetes namespace_enabled True
-    fi
-    if is_service_enabled tempest && [[ "$KURYR_K8S_SERIAL_TESTS" == "True" ]]; then
-        iniset $TEMPEST_CONFIG kuryr_kubernetes run_tests_serial True
-    fi
-    if is_service_enabled tempest && [[ "$KURYR_MULTI_VIF_DRIVER" == "npwg_multiple_interfaces" ]]; then
-        iniset $TEMPEST_CONFIG kuryr_kubernetes npwg_multi_vif_enabled True
+    if is_service_enabled tempest; then
+        update_tempest_conf_file "$use_octavia"
     fi
 fi
 
@@ -999,16 +1134,6 @@ if [[ "$1" == "unstack" ]]; then
         # when doing stack.sh again, openshift-node will use old certificates.
         sudo rm -rf ${OPENSHIFT_DATA_DIR}
     fi
-    if is_service_enabled legacy_etcd; then
-        stop_container etcd
-    fi
 
     cleanup_kuryr_devstack_iptables
 fi
-
-if [[ "$1" == "clean" ]]; then
-    if is_service_enabled legacy_etcd; then
-        # Cleanup Etcd for the next stacking
-        sudo rm -rf "$KURYR_ETCD_DATA_DIR"
-    fi
-fi
