@@ -16,6 +16,9 @@
 K8S_API_BASE = '/api/v1'
 K8S_API_NAMESPACES = K8S_API_BASE + '/namespaces'
 K8S_API_CRD = '/apis/openstack.org/v1'
+K8S_API_CRD_NAMESPACES = K8S_API_CRD + '/namespaces'
+K8S_API_CRD_KURYRNETS = K8S_API_CRD + '/kuryrnets'
+K8S_API_CRD_KURYRNETPOLICIES = K8S_API_CRD + '/kuryrnetpolicies'
 K8S_API_POLICIES = '/apis/networking.k8s.io/v1/networkpolicies'
 
 K8S_API_NPWG_CRD = '/apis/k8s.cni.cncf.io/v1'
@@ -27,14 +30,17 @@ K8S_OBJ_ENDPOINTS = 'Endpoints'
 K8S_OBJ_POLICY = 'NetworkPolicy'
 
 K8S_OBJ_KURYRNET = 'KuryrNet'
+K8S_OBJ_KURYRNETPOLICY = 'KuryrNetPolicy'
 
 K8S_POD_STATUS_PENDING = 'Pending'
 
 K8S_ANNOTATION_PREFIX = 'openstack.org/kuryr'
 K8S_ANNOTATION_VIF = K8S_ANNOTATION_PREFIX + '-vif'
+K8S_ANNOTATION_LABEL = K8S_ANNOTATION_PREFIX + '-pod-label'
 K8S_ANNOTATION_LBAAS_SPEC = K8S_ANNOTATION_PREFIX + '-lbaas-spec'
 K8S_ANNOTATION_LBAAS_STATE = K8S_ANNOTATION_PREFIX + '-lbaas-state'
 K8S_ANNOTATION_NET_CRD = K8S_ANNOTATION_PREFIX + '-net-crd'
+K8S_ANNOTATION_NETPOLICY_CRD = K8S_ANNOTATION_PREFIX + '-netpolicy-crd'
 K8S_ANNOTATION_LBAAS_RT_STATE = K8S_ANNOTATION_PREFIX + '-lbaas-route-state'
 K8S_ANNOTATION_LBAAS_RT_NOTIF = K8S_ANNOTATION_PREFIX + '-lbaas-route-notif'
 K8S_ANNOTATION_ROUTE_STATE = K8S_ANNOTATION_PREFIX + '-route-state'
@@ -65,3 +71,9 @@ VIF_POOL_SHOW = '/showPool'
 DEFAULT_IFNAME = 'eth0'
 
 ADDITIONAL_IFNAME_PREFIX = 'eth'
+K8S_NPWG_SRIOV_PREFIX = "intel.com/sriov"
+
+K8S_OPERATOR_IN = 'in'
+K8S_OPERATOR_NOT_IN = 'notin'
+K8S_OPERATOR_DOES_NOT_EXIST = 'doesnotexist'
+K8S_OPERATOR_EXISTS = 'exists'
