@@ -51,21 +51,34 @@ class DriverBase(object):
     """
 
     @classmethod
-    def get_instance(cls, driver_alias=None):
-        """Get an implementing driver instance."""
+    def get_instance(cls, specific_driver=None, scope='default'):
+        """Get an implementing driver instance.
+
+        :param specific_driver: Loads a specific driver instead of using conf.
+                                Uses separate manager entry so that loading of
+                                default/other drivers is not affected.
+        :param scope: Loads the driver in the given scope (if independent
+                      instances of a driver are required)
+        """
 
         alias = cls.ALIAS
-        driver_name = alias + '_driver' if not driver_alias else driver_alias
+
+        if specific_driver:
+            driver_key = '{}:{}:{}'.format(alias, specific_driver, scope)
+        else:
+            driver_key = '{}:_from_cfg:{}'.format(alias, scope)
+
         try:
-            manager = _DRIVER_MANAGERS[driver_name]
+            manager = _DRIVER_MANAGERS[driver_key]
         except KeyError:
-            name = (config.CONF.kubernetes[driver_name] if not driver_alias
-                    else driver_alias)
+            driver_name = (specific_driver or
+                           config.CONF.kubernetes[alias + '_driver'])
+
             manager = stv_driver.DriverManager(
                 namespace="%s.%s" % (_DRIVER_NAMESPACE_BASE, alias),
-                name=name,
+                name=driver_name,
                 invoke_on_load=True)
-            _DRIVER_MANAGERS[driver_name] = manager
+            _DRIVER_MANAGERS[driver_key] = manager
 
         driver = manager.driver
         if not isinstance(driver, cls):
@@ -366,6 +379,20 @@ class PodVIFDriver(DriverBase):
         """
         raise NotImplementedError()
 
+    @abc.abstractmethod
+    def update_vif_sgs(self, pod, security_groups):
+        """Update VIF security groups.
+
+        Implementing drivers should update the port associated to the pod
+        with the specified security groups.
+
+        :param pod: dict containing Kubernetes Pod object
+        :param security_groups: list containing security groups' IDs as
+                                returned by
+                                `PodSecurityGroupsDriver.get_security_groups`
+        """
+        raise NotImplementedError()
+
 
 @six.add_metaclass(abc.ABCMeta)
 class MultiVIFDriver(DriverBase):
@@ -425,7 +452,7 @@ class LBaaSDriver(DriverBase):
 
     @abc.abstractmethod
     def ensure_loadbalancer(self, name, project_id, subnet_id, ip,
-                            security_groups_ids, service_type):
+                            security_groups_ids, service_type, provider):
         """Get or create load balancer.
 
         :param name: LoadBlancer name
@@ -435,6 +462,7 @@ class LBaaSDriver(DriverBase):
         :param security_groups_ids: security groups that should be allowed
                                     access to the load balancer
         :param service_type: K8s service type (ClusterIP or LoadBalancer)
+        :param provider: load balancer backend service
         """
         raise NotImplementedError()
 
@@ -653,12 +681,14 @@ class ServicePubIpDriver(DriverBase):
     ALIAS = 'service_public_ip'
 
     @abc.abstractmethod
-    def acquire_service_pub_ip_info(self, spec_type, spec_lb_ip, project_id):
+    def acquire_service_pub_ip_info(self, spec_type, spec_lb_ip, project_id,
+                                    port_id_to_be_associated=None):
         """Get k8s service loadbalancer IP info based on service spec
 
         :param spec_type: service.spec.type field
         :param spec_lb_ip: service spec LoadBlaceIP field
         :param project_id: openstack project id
+        :param port_id_to_be_associated: port id to associate
 
         """
         raise NotImplementedError()
@@ -704,15 +734,62 @@ class NetworkPolicyDriver(DriverBase):
 
         :param policy: dict containing Kubernetes NP object
         :param project_id: openstack project_id
+        :returns: list of Pod objects affected by the network policy
+        creation or its podSelector modification
         """
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def release_network_policy(self, policy, project_id):
+    def release_network_policy(self, kuryrnetpolicy):
         """Delete a network policy
 
+        :param kuryrnetpolicy: dict containing Kuryrnetpolicy CRD object
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def affected_pods(self, policy, selector=None):
+        """Return affected pods by the policy
+
+        This method returns the list of pod objects affected by the policy, or
+        by the selector if it is specified.
+
         :param policy: dict containing Kubernetes NP object
-        :param project_id
+        :param selector: (optional) specifc pod selector
+        :returns: list of Pods objects affected by the policy or the selector
+                  if it is pased
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def knps_on_namespace(self, namespace):
+        """Check if there si kuryr network policy CRDs on the namespace
+
+        This method returns true if there are knps on the specified namespace
+        or false otherwise
+
+        :param namespace: namespace name where the knps CRDs should be
+        :returns: true if knps CRDs on the namespace, false otherwise
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def namespaced_pods(self, policy):
+        """Return pods on the policy namespace
+
+        This method returns the pods on the network policy namespace
+
+        :param policy: dict containing Kubernetes NP object
+        :returns: list of Pods objects on the policy namespace
+        """
+        raise NotImplementedError()
+
+    @abc.abstractmethod
+    def get_kuryrnetpolicy_crd(self, policy):
+        """Return kuryrnetpolicy CRD object associated to the policy
+
+        :param policy: dict containing Kubernetes NP object
+        :returns: kuryrnetpolicy CRD object associated to the policy
         """
         raise NotImplementedError()
 
