@@ -14,14 +14,16 @@
 #    under the License.
 
 import random
+import six
+import sys
 import time
+import traceback
 
 import requests
 
 from neutronclient.common import exceptions as n_exc
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import excutils
 from oslo_utils import timeutils
 
 from kuryr_kubernetes import clients
@@ -34,8 +36,22 @@ CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 _ACTIVATION_TIMEOUT = CONF.neutron_defaults.lbaas_activation_timeout
-_SUPPORTED_LISTENER_PROT = ('HTTP', 'HTTPS', 'TCP')
+
+_PROVIDER_SUPPORTED_LISTENER_PROT = {
+    'amphora': ['HTTP', 'HTTPS', 'TCP', 'UDP'],
+    'ovn': ['TCP', 'UDP'],
+    'haproxy':  ['HTTP', 'HTTPS', 'TCP']}
+
 _L7_POLICY_ACT_REDIRECT_TO_POOL = 'REDIRECT_TO_POOL'
+# NOTE(yboaron):Prior to sending create request to Octavia, LBaaS driver
+# verifies that LB is in a stable state by polling LB's provisioning_status
+# using backoff timer.
+# A similar method is used also for the delete flow.
+# Unlike LB creation, rest of octavia operations are completed usually after
+# few seconds. Next constants define the intervals values for 'fast' and
+# 'slow' (will be used for LB creation)  polling.
+_LB_STS_POLL_FAST_INTERVAL = 1
+_LB_STS_POLL_SLOW_INTERVAL = 3
 
 
 class LBaaSv2Driver(base.LBaaSDriver):
@@ -53,10 +69,11 @@ class LBaaSv2Driver(base.LBaaSDriver):
         return "%s/%s/%s" % (loadbalancer.name, namespace, svc_name)
 
     def ensure_loadbalancer(self, name, project_id, subnet_id, ip,
-                            security_groups_ids=None, service_type=None):
+                            security_groups_ids=None, service_type=None,
+                            provider=None):
         request = obj_lbaas.LBaaSLoadBalancer(
             name=name, project_id=project_id, subnet_id=subnet_id, ip=ip,
-            security_groups=security_groups_ids)
+            security_groups=security_groups_ids, provider=provider)
         response = self._ensure(request, self._create_loadbalancer,
                                 self._find_loadbalancer)
         if not response:
@@ -88,13 +105,17 @@ class LBaaSv2Driver(base.LBaaSDriver):
             self._release(loadbalancer, loadbalancer,
                           lbaas.delete_loadbalancer, loadbalancer.id)
 
-            sg_id = self._find_listeners_sg(loadbalancer)
-            if sg_id:
-                try:
-                    neutron.delete_security_group(sg_id)
-                except n_exc.NeutronClientException:
-                    LOG.exception('Error when deleting loadbalancer security '
-                                  'group. Leaving it orphaned.')
+        sg_id = self._find_listeners_sg(loadbalancer)
+        if sg_id:
+            # Note: reusing activation timeout as deletion timeout
+            self._wait_for_deletion(loadbalancer, _ACTIVATION_TIMEOUT)
+            try:
+                neutron.delete_security_group(sg_id)
+            except n_exc.NeutronClientException:
+                LOG.exception('Error when deleting loadbalancer security '
+                              'group. Leaving it orphaned.')
+            except n_exc.NotFound:
+                LOG.debug('Security group %s already deleted', sg_id)
 
     def _ensure_security_groups(self, loadbalancer, service_type):
         # We only handle SGs for legacy LBaaSv2, Octavia handles it dynamically
@@ -149,46 +170,47 @@ class LBaaSv2Driver(base.LBaaSDriver):
                     LOG.exception('Failed when creating security group rule '
                                   'for listener %s.', listener.name)
 
-    def _remove_default_octavia_rules(self, sg_id, listener):
-        neutron = clients.get_neutron_client()
-        for remaining in self._provisioning_timer(_ACTIVATION_TIMEOUT):
-            listener_rules = neutron.list_security_group_rules(
-                security_group_id=sg_id,
-                protocol=listener.protocol,
-                port_range_min=listener.port,
-                port_range_max=listener.port,
-                direction='ingress')
-            for rule in listener_rules['security_group_rules']:
-                if not (rule.get('remote_group_id') or
-                        rule.get('remote_ip_prefix')):
-                    # remove default sg rules
-                    neutron.delete_security_group_rule(rule['id'])
-                    return
-
     def _extend_lb_security_group_rules(self, loadbalancer, listener):
         neutron = clients.get_neutron_client()
-        sg_id = self._get_vip_port(loadbalancer).get('security_groups')[0]
 
-        # wait until octavia adds default sg rules
-        self._remove_default_octavia_rules(sg_id, listener)
+        if CONF.octavia_defaults.sg_mode == 'create':
+            sg_id = self._find_listeners_sg(loadbalancer)
+            # if an SG for the loadbalancer has not being created, create one
+            if not sg_id:
+                sg = neutron.create_security_group({
+                    'security_group': {
+                        'name': loadbalancer.name,
+                        'project_id': loadbalancer.project_id,
+                        },
+                    })
+                sg_id = sg['security_group']['id']
+                loadbalancer.security_groups.append(sg_id)
+                vip_port = self._get_vip_port(loadbalancer)
+                neutron.update_port(
+                    vip_port.get('id'),
+                    {'port': {
+                        'security_groups': loadbalancer.security_groups}})
+        else:
+            sg_id = self._get_vip_port(loadbalancer).get('security_groups')[0]
 
         for sg in loadbalancer.security_groups:
-            try:
-                neutron.create_security_group_rule({
-                    'security_group_rule': {
-                        'direction': 'ingress',
-                        'port_range_min': listener.port,
-                        'port_range_max': listener.port,
-                        'protocol': listener.protocol,
-                        'security_group_id': sg_id,
-                        'remote_group_id': sg,
-                        'description': listener.name,
-                    },
-                })
-            except n_exc.NeutronClientException as ex:
-                if ex.status_code != requests.codes.conflict:
-                    LOG.exception('Failed when creating security group rule '
-                                  'for listener %s.', listener.name)
+            if sg != sg_id:
+                try:
+                    neutron.create_security_group_rule({
+                        'security_group_rule': {
+                            'direction': 'ingress',
+                            'port_range_min': listener.port,
+                            'port_range_max': listener.port,
+                            'protocol': listener.protocol,
+                            'security_group_id': sg_id,
+                            'remote_group_id': sg,
+                            'description': listener.name,
+                        },
+                    })
+                except n_exc.NeutronClientException as ex:
+                    if ex.status_code != requests.codes.conflict:
+                        LOG.exception('Failed when creating security group '
+                                      'rule for listener %s.', listener.name)
 
         # ensure routes have access to the services
         service_subnet_cidr = self._get_subnet_cidr(loadbalancer.subnet_id)
@@ -230,29 +252,46 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
     def _ensure_security_group_rules(self, loadbalancer, listener,
                                      service_type):
-        namespace_isolation = (
-            'namespace' in CONF.kubernetes.enabled_handlers and
-            CONF.kubernetes.service_security_groups_driver == 'namespace')
         if loadbalancer.provider == const.NEUTRON_LBAAS_HAPROXY_PROVIDER:
             self._ensure_lb_security_group_rule(loadbalancer, listener)
-        elif service_type == 'ClusterIP' and namespace_isolation:
+        elif service_type == 'ClusterIP':
             self._extend_lb_security_group_rules(loadbalancer, listener)
 
     def ensure_listener(self, loadbalancer, protocol, port,
                         service_type='ClusterIP'):
-        if protocol not in _SUPPORTED_LISTENER_PROT:
-            LOG.info("Protocol: %(prot)s: is not supported by LBaaSV2", {
-                'prot': protocol})
+
+        # NOTE(yboaron): Since retrieving Octavia capabilities/version is not
+        # supported via  the OpenstackSdk, the list of allowed listener's
+        # protocols will be defined statically.
+        # Kuryr still need to handle the case in which listener's protocol
+        # (e.g: UDP) is not supported by Octavia.
+        provider = loadbalancer.provider or 'amphora'
+        try:
+            if protocol not in _PROVIDER_SUPPORTED_LISTENER_PROT[provider]:
+                LOG.info("Protocol: %(prot)s: is not supported by "
+                         "%(provider)s",
+                         {'prot': protocol, 'provider': provider})
+                return None
+        except KeyError:
+            LOG.info("Provider %(provider)s doesnt exist in "
+                     "_PROVIDER_SUPPORTED_LISTENER_PROT",
+                     {'provider': provider})
             return None
+
         name = "%s:%s:%s" % (loadbalancer.name, protocol, port)
         listener = obj_lbaas.LBaaSListener(name=name,
                                            project_id=loadbalancer.project_id,
                                            loadbalancer_id=loadbalancer.id,
                                            protocol=protocol,
                                            port=port)
-        result = self._ensure_provisioned(loadbalancer, listener,
-                                          self._create_listener,
-                                          self._find_listener)
+        try:
+            result = self._ensure_provisioned(
+                loadbalancer, listener, self._create_listener,
+                self._find_listener, _LB_STS_POLL_SLOW_INTERVAL)
+        except n_exc.BadRequest:
+            LOG.info("Listener creation failed, most probably because "
+                     "protocol %(prot)s is not supported", {'prot': protocol})
+            return None
 
         self._ensure_security_group_rules(loadbalancer, result, service_type)
 
@@ -265,10 +304,7 @@ class LBaaSv2Driver(base.LBaaSDriver):
                       lbaas.delete_listener,
                       listener.id)
 
-        if loadbalancer.provider == const.NEUTRON_LBAAS_HAPROXY_PROVIDER:
-            sg_id = self._find_listeners_sg(loadbalancer)
-        else:
-            sg_id = self._get_vip_port(loadbalancer).get('security_groups')[0]
+        sg_id = self._find_listeners_sg(loadbalancer)
         if sg_id:
             rules = neutron.list_security_group_rules(
                 security_group_id=sg_id, description=listener.name)
@@ -355,13 +391,25 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
     def _create_loadbalancer(self, loadbalancer):
         lbaas = clients.get_loadbalancer_client()
-        response = lbaas.create_loadbalancer({'loadbalancer': {
+
+        request = {'loadbalancer': {
             'name': loadbalancer.name,
             'project_id': loadbalancer.project_id,
             'vip_address': str(loadbalancer.ip),
-            'vip_subnet_id': loadbalancer.subnet_id}})
+            'vip_subnet_id': loadbalancer.subnet_id}}
+
+        if loadbalancer.provider is not None:
+            request['loadbalancer']['provider'] = loadbalancer.provider
+
+        response = lbaas.create_loadbalancer(request)
         loadbalancer.id = response['loadbalancer']['id']
         loadbalancer.port_id = self._get_vip_port(loadbalancer).get("id")
+        if (loadbalancer.provider is not None and
+                loadbalancer.provider != response['loadbalancer']['provider']):
+            LOG.error("Request provider(%s) != Response provider(%s)",
+                      loadbalancer.provider,
+                      response['loadbalancer']['provider'])
+            return None
         loadbalancer.provider = response['loadbalancer']['provider']
         return loadbalancer
 
@@ -424,8 +472,15 @@ class LBaaSv2Driver(base.LBaaSDriver):
             pool.id = response['pool']['id']
             return pool
         except n_exc.StateInvalidClient:
-            with excutils.save_and_reraise_exception():
+            (type_, value, tb) = sys.exc_info()
+            try:
                 self._cleanup_bogus_pool(lbaas, pool, lb_algorithm)
+            except Exception:
+                LOG.error('Pool creation traceback: %s',
+                          traceback.format_exception(type_, value, tb))
+                raise
+            else:
+                six.reraise(type_, value, tb)
 
     def _cleanup_bogus_pool(self, lbaas, pool, lb_algorithm):
         # REVISIT(ivc): LBaaSv2 creates pool object despite raising an
@@ -510,9 +565,11 @@ class LBaaSv2Driver(base.LBaaSDriver):
                 LOG.debug("Found %(obj)s", {'obj': result})
         return result
 
-    def _ensure_provisioned(self, loadbalancer, obj, create, find):
-        for remaining in self._provisioning_timer(_ACTIVATION_TIMEOUT):
-            self._wait_for_provisioning(loadbalancer, remaining)
+    def _ensure_provisioned(self, loadbalancer, obj, create, find,
+                            interval=_LB_STS_POLL_FAST_INTERVAL):
+        for remaining in self._provisioning_timer(_ACTIVATION_TIMEOUT,
+                                                  interval):
+            self._wait_for_provisioning(loadbalancer, remaining, interval)
             try:
                 result = self._ensure(obj, create, find)
                 if result:
@@ -535,10 +592,11 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         raise k_exc.ResourceNotReady(obj)
 
-    def _wait_for_provisioning(self, loadbalancer, timeout):
+    def _wait_for_provisioning(self, loadbalancer, timeout,
+                               interval=_LB_STS_POLL_FAST_INTERVAL):
         lbaas = clients.get_loadbalancer_client()
 
-        for remaining in self._provisioning_timer(timeout):
+        for remaining in self._provisioning_timer(timeout, interval):
             response = lbaas.show_loadbalancer(loadbalancer.id)
             status = response['loadbalancer']['provisioning_status']
             if status == 'ACTIVE':
@@ -553,9 +611,19 @@ class LBaaSv2Driver(base.LBaaSDriver):
 
         raise k_exc.ResourceNotReady(loadbalancer)
 
-    def _provisioning_timer(self, timeout):
+    def _wait_for_deletion(self, loadbalancer, timeout,
+                           interval=_LB_STS_POLL_FAST_INTERVAL):
+        lbaas = clients.get_loadbalancer_client()
+
+        for remaining in self._provisioning_timer(timeout, interval):
+            try:
+                lbaas.show_loadbalancer(loadbalancer.id)
+            except n_exc.NotFound:
+                return
+
+    def _provisioning_timer(self, timeout,
+                            interval=_LB_STS_POLL_FAST_INTERVAL):
         # REVISIT(ivc): consider integrating with Retry
-        interval = 3
         max_interval = 15
         with timeutils.StopWatch(duration=timeout) as timer:
             while not timer.expired():
