@@ -81,6 +81,14 @@ MEMOIZE = cache.get_memoization_decorator(
 
 cache.configure_cache_region(oslo_cfg.CONF, node_driver_cache_region)
 
+VIF_TYPE_TO_DRIVER_MAPPING = {
+    'VIFOpenVSwitch': 'neutron-vif',
+    'VIFBridge': 'neutron-vif',
+    'VIFVlanNested': 'nested-vlan',
+    'VIFMacvlanNested': 'nested-macvlan',
+    'VIFSriov': 'sriov'
+}
+
 
 class NoopVIFPool(base.VIFPoolDriver):
     """No pool VIFs for Kubernetes Pods"""
@@ -98,6 +106,9 @@ class NoopVIFPool(base.VIFPoolDriver):
     def activate_vif(self, pod, vif):
         self._drv_vif.activate_vif(pod, vif)
 
+    def update_vif_sgs(self, pod, sgs):
+        self._drv_vif.update_vif_sgs(pod, sgs)
+
     def sync_pools(self):
         pass
 
@@ -141,6 +152,9 @@ class BaseVIFPool(base.VIFPoolDriver):
     def activate_vif(self, pod, vif):
         self._drv_vif.activate_vif(pod, vif)
 
+    def update_vif_sgs(self, pod, sgs):
+        self._drv_vif.update_vif_sgs(pod, sgs)
+
     def _get_pool_size(self, pool_key=None):
         return len(self._available_ports_pools.get(pool_key, []))
 
@@ -415,7 +429,7 @@ class NeutronVIFPool(BaseVIFPool):
 
     def delete_network_pools(self, net_id):
         if not hasattr(self, '_available_ports_pools'):
-            LOG.info("Kuryr-controller not yet ready to delete network pools"
+            LOG.info("Kuryr-controller not yet ready to delete network "
                      "pools.")
             raise exceptions.ResourceNotReady(net_id)
         neutron = clients.get_neutron_client()
@@ -743,7 +757,7 @@ class NestedVIFPool(BaseVIFPool):
 
     def delete_network_pools(self, net_id):
         if not hasattr(self, '_available_ports_pools'):
-            LOG.info("Kuryr-controller not yet ready to delete network pools"
+            LOG.info("Kuryr-controller not yet ready to delete network "
                      "pools.")
             raise exceptions.ResourceNotReady(net_id)
         neutron = clients.get_neutron_client()
@@ -806,9 +820,10 @@ class MultiVIFPool(base.VIFPoolDriver):
                 LOG.error("The pool and pod driver selected are not "
                           "compatible. They will be skipped")
                 raise exceptions.MultiPodDriverPoolConfigurationNotSupported()
-            drv_vif = base.PodVIFDriver.get_instance(driver_alias=pod_driver)
+            drv_vif = base.PodVIFDriver.get_instance(
+                specific_driver=pod_driver)
             drv_pool = base.VIFPoolDriver.get_instance(
-                driver_alias=pool_driver)
+                specific_driver=pool_driver)
             drv_pool.set_vif_driver(drv_vif)
             self._vif_drvs[pod_driver] = drv_pool
 
@@ -818,12 +833,16 @@ class MultiVIFPool(base.VIFPoolDriver):
             pod, project_id, subnets, security_groups)
 
     def release_vif(self, pod, vif, *argv):
-        pod_vif_type = self._get_pod_vif_type(pod)
-        self._vif_drvs[pod_vif_type].release_vif(pod, vif, *argv)
+        vif_drv_alias = self._get_vif_drv_alias(vif)
+        self._vif_drvs[vif_drv_alias].release_vif(pod, vif, *argv)
 
     def activate_vif(self, pod, vif):
+        vif_drv_alias = self._get_vif_drv_alias(vif)
+        self._vif_drvs[vif_drv_alias].activate_vif(pod, vif)
+
+    def update_vif_sgs(self, pod, sgs):
         pod_vif_type = self._get_pod_vif_type(pod)
-        self._vif_drvs[pod_vif_type].activate_vif(pod, vif)
+        self._vif_drvs[pod_vif_type].update_vif_sgs(pod, sgs)
 
     def delete_network_pools(self, net_id):
         for vif_drv in self._vif_drvs.values():
@@ -851,3 +870,7 @@ class MultiVIFPool(base.VIFPoolDriver):
                                  oslo_cfg.CONF.kubernetes.pod_vif_driver)
             return pod_vif
         return oslo_cfg.CONF.kubernetes.pod_vif_driver
+
+    def _get_vif_drv_alias(self, vif):
+        vif_type_name = type(vif).__name__
+        return VIF_TYPE_TO_DRIVER_MAPPING[vif_type_name]
