@@ -50,7 +50,7 @@ class LBaaSSpecHandler(k8s_base.ResourceEventHandler):
         lbaas_spec = self._get_lbaas_spec(service)
 
         if self._should_ignore(service):
-            LOG.debug("Skiping Kubernetes service of an unsupported kind or"
+            LOG.debug("Skiping Kubernetes service of an unsupported kind or "
                       "without a selector as Kubernetes does not create an "
                       "endpoint object for it.")
             return
@@ -229,6 +229,15 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
         self._drv_pod_project = drv_base.PodProjectDriver.get_instance()
         self._drv_pod_subnets = drv_base.PodSubnetsDriver.get_instance()
         self._drv_service_pub_ip = drv_base.ServicePubIpDriver.get_instance()
+        # Note(yboaron) LBaaS driver supports 'provider' parameter in
+        # Load Balancer creation flow.
+        # We need to set the requested load balancer provider
+        # according to 'endpoints_driver_octavia_provider' configuration.
+        self._lb_provider = None
+        if (config.CONF.kubernetes.endpoints_driver_octavia_provider
+                != 'default'):
+            self._lb_provider = (
+                config.CONF.kubernetes.endpoints_driver_octavia_provider)
 
     def on_present(self, endpoints):
         lbaas_spec = self._get_lbaas_spec(endpoints)
@@ -352,8 +361,7 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                                                              p.port]
             except KeyError:
                 continue
-        current_targets = {(str(m.ip), m.port, m.pool_id)
-                           for m in lbaas_state.members}
+        current_targets = {(str(m.ip), m.port) for m in lbaas_state.members}
 
         for subset in endpoints.get('subsets', []):
             subset_ports = subset.get('ports', [])
@@ -369,14 +377,14 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                     continue
                 for subset_port in subset_ports:
                     target_port = subset_port['port']
+                    if (target_ip, target_port) in current_targets:
+                        continue
                     port_name = subset_port.get('name')
                     try:
                         pool = pool_by_tgt_name[port_name]
                     except KeyError:
                         LOG.debug("No pool found for port: %r", port_name)
                         continue
-                    if (target_ip, target_port, pool.id) in current_targets:
-                        continue
                     # TODO(apuimedo): Do not pass subnet_id at all when in
                     # L3 mode once old neutron-lbaasv2 is not supported, as
                     # octavia does not require it
@@ -409,49 +417,25 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                             'namespace': target_ref['namespace']}}
         project_id = self._drv_pod_project.get_project(pod)
         subnets_map = self._drv_pod_subnets.get_subnets(pod, project_id)
-        subnet_ids = [subnet_id for subnet_id, network in subnets_map.items()
-                      for subnet in network.subnets.objects
-                      if ip in subnet.cidr]
-        if subnet_ids:
-            return subnet_ids[0]
-        else:
-            # NOTE(ltomasbo): We are assuming that if ip is not on the
-            # pod subnet is because the member is using hostnetworking. In
-            # this worker_nodes_subnet will be used
-            return config.CONF.pod_vif_nested.worker_nodes_subnet
-
-    def _get_port_in_pool(self, pool, lbaas_state, lbaas_spec):
-        for l in lbaas_state.listeners:
-            if l.id != pool.listener_id:
-                continue
-            for port in lbaas_spec.ports:
-                if l.port == port.port and l.protocol == port.protocol:
-                    return port
-        return None
+        # FIXME(ivc): potentially unsafe [0] index
+        return [subnet_id for subnet_id, network in subnets_map.items()
+                for subnet in network.subnets.objects
+                if ip in subnet.cidr][0]
 
     def _remove_unused_members(self, endpoints, lbaas_state, lbaas_spec):
-        spec_ports = {}
-        for pool in lbaas_state.pools:
-            port = self._get_port_in_pool(pool, lbaas_state, lbaas_spec)
-            if port:
-                spec_ports[port.name] = pool.id
-
-        current_targets = {(a['ip'], p['port'],
-                            spec_ports.get(p.get('name')))
+        spec_port_names = {p.name for p in lbaas_spec.ports}
+        current_targets = {(a['ip'], p['port'])
                            for s in endpoints['subsets']
                            for a in s['addresses']
                            for p in s['ports']
-                           if p.get('name') in spec_ports}
-
+                           if p.get('name') in spec_port_names}
         removed_ids = set()
         for member in lbaas_state.members:
-            if ((str(member.ip), member.port, member.pool_id) in
-                    current_targets):
+            if (str(member.ip), member.port) in current_targets:
                 continue
             self._drv_lbaas.release_member(lbaas_state.loadbalancer,
                                            member)
             removed_ids.add(member.id)
-
         if removed_ids:
             lbaas_state.members = [m for m in lbaas_state.members
                                    if m.id not in removed_ids]
@@ -569,7 +553,7 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
         k8s = clients.get_kubernetes_client()
         svc_link = self._get_service_link(endpoints)
         try:
-            k8s.patch_status(svc_link, status_data)
+            k8s.patch("status", svc_link, status_data)
         except k_exc.K8sClientException:
             # REVISIT(ivc): only raise ResourceNotReady for NotFound
             raise k_exc.ResourceNotReady(svc_link)
@@ -611,7 +595,8 @@ class LoadBalancerHandler(k8s_base.ResourceEventHandler):
                     subnet_id=lbaas_spec.subnet_id,
                     ip=lbaas_spec.ip,
                     security_groups_ids=lbaas_spec.security_groups_ids,
-                    service_type=lbaas_spec.type)
+                    service_type=lbaas_spec.type,
+                    provider=self._lb_provider)
                 changed = True
             elif lbaas_state.service_pub_ip_info:
                 self._drv_service_pub_ip.release_pub_ip(
