@@ -98,7 +98,25 @@ class TestLBaaSv2Driver(test_base.TestCase):
             lbaas.lbaas_loadbalancer_path % loadbalancer.id,
             params={'cascade': True})
 
-    def test_ensure_listener(self):
+    def test_ensure_listener_tcp(self):
+        self._test_ensure_listener('TCP')
+
+    def test_ensure_listener_udp(self):
+        self._test_ensure_listener('UDP')
+
+    def test_ensure_listener_unsupported_protocol(self):
+        self._test_ensure_listener('NOT_SUPPORTED')
+
+    def test_ensure_listener_ovn_tcp(self):
+        self._test_ensure_listener('TCP', 'ovn')
+
+    def test_ensure_listener_ovn_udp(self):
+        self._test_ensure_listener('UDP', 'ovn')
+
+    def test_ensure_listener_ovn_unsupported_protocol(self):
+        self._test_ensure_listener('HTTP', 'ovn')
+
+    def _test_ensure_listener(self, protocol, provider=None):
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
         expected_resp = mock.sentinel.expected_resp
@@ -107,21 +125,27 @@ class TestLBaaSv2Driver(test_base.TestCase):
         subnet_id = 'D3FA400A-F543-4B91-9CD3-047AF0CE42D1'
         ip = '1.2.3.4'
         loadbalancer_id = '00EE9E11-91C2-41CF-8FD4-7970579E5C4C'
-        protocol = 'TCP'
         port = 1234
         loadbalancer = obj_lbaas.LBaaSLoadBalancer(
             id=loadbalancer_id, name=name, project_id=project_id,
-            subnet_id=subnet_id, ip=ip)
+            subnet_id=subnet_id, ip=ip, provider=provider)
         # TODO(ivc): handle security groups
         m_driver._ensure_provisioned.return_value = expected_resp
 
         resp = cls.ensure_listener(m_driver, loadbalancer,
                                    protocol, port)
 
+        provider = loadbalancer.provider or 'amphora'
+        if (protocol not in
+                d_lbaasv2._PROVIDER_SUPPORTED_LISTENER_PROT[provider]):
+            self.assertIsNone(resp)
+            return
+
         m_driver._ensure_provisioned.assert_called_once_with(
             loadbalancer, mock.ANY, m_driver._create_listener,
-            m_driver._find_listener)
+            m_driver._find_listener, d_lbaasv2._LB_STS_POLL_SLOW_INTERVAL)
         listener = m_driver._ensure_provisioned.call_args[0][1]
+
         self.assertEqual("%s:%s:%s" % (loadbalancer.name, protocol, port),
                          listener.name)
         self.assertEqual(project_id, listener.project_id)
@@ -130,6 +154,26 @@ class TestLBaaSv2Driver(test_base.TestCase):
         self.assertEqual(port, listener.port)
         self.assertEqual(expected_resp, resp)
 
+    def test_ensure_listener_bad_request_exception(self):
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        name = 'TEST_NAME'
+        project_id = 'TEST_PROJECT'
+        subnet_id = 'D3FA400A-F543-4B91-9CD3-047AF0CE42D1'
+        ip = '1.2.3.4'
+        loadbalancer_id = '00EE9E11-91C2-41CF-8FD4-7970579E5C4C'
+        port = 1234
+        protocol = 'TCP'
+        provider = 'amphora'
+        loadbalancer = obj_lbaas.LBaaSLoadBalancer(
+            id=loadbalancer_id, name=name, project_id=project_id,
+            subnet_id=subnet_id, ip=ip, provider=provider)
+        m_driver._ensure_provisioned.side_effect = n_exc.BadRequest
+
+        resp = cls.ensure_listener(m_driver, loadbalancer,
+                                   protocol, port)
+        self.assertIsNone(resp)
+
     def test_release_listener(self):
         neutron = self.useFixture(k_fix.MockNeutronClient()).client
         lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
@@ -137,8 +181,6 @@ class TestLBaaSv2Driver(test_base.TestCase):
             'security_group_rules': []}
         cls = d_lbaasv2.LBaaSv2Driver
         m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-        m_driver._get_vip_port.return_value = {
-            'security_groups': [mock.sentinel.sg_id]}
         loadbalancer = mock.Mock()
         listener = mock.Mock()
 
@@ -258,6 +300,59 @@ class TestLBaaSv2Driver(test_base.TestCase):
                              getattr(ret, attr))
         self.assertEqual(loadbalancer_id, ret.id)
 
+    def test_create_loadbalancer_provider_defined(self):
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        loadbalancer = obj_lbaas.LBaaSLoadBalancer(
+            name='TEST_NAME', project_id='TEST_PROJECT', ip='1.2.3.4',
+            subnet_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1',
+            security_groups=[],
+            provider='amphora')
+        loadbalancer_id = '00EE9E11-91C2-41CF-8FD4-7970579E5C4C'
+        req = {'loadbalancer': {
+            'name': loadbalancer.name,
+            'project_id': loadbalancer.project_id,
+            'vip_address': str(loadbalancer.ip),
+            'vip_subnet_id': loadbalancer.subnet_id,
+            'provider': loadbalancer.provider,
+        }}
+        resp = {'loadbalancer': {'id': loadbalancer_id, 'provider': 'amphora'}}
+        lbaas.create_loadbalancer.return_value = resp
+        m_driver._get_vip_port.return_value = {'id': mock.sentinel.port_id}
+
+        ret = cls._create_loadbalancer(m_driver, loadbalancer)
+        lbaas.create_loadbalancer.assert_called_once_with(req)
+        for attr in loadbalancer.obj_fields:
+            self.assertEqual(getattr(loadbalancer, attr),
+                             getattr(ret, attr))
+        self.assertEqual(loadbalancer_id, ret.id)
+
+    def test_create_loadbalancer_provider_mismatch(self):
+        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
+        cls = d_lbaasv2.LBaaSv2Driver
+        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
+        loadbalancer = obj_lbaas.LBaaSLoadBalancer(
+            name='TEST_NAME', project_id='TEST_PROJECT', ip='1.2.3.4',
+            subnet_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1',
+            security_groups=[],
+            provider='amphora')
+        loadbalancer_id = '00EE9E11-91C2-41CF-8FD4-7970579E5C4C'
+        req = {'loadbalancer': {
+            'name': loadbalancer.name,
+            'project_id': loadbalancer.project_id,
+            'vip_address': str(loadbalancer.ip),
+            'vip_subnet_id': loadbalancer.subnet_id,
+            'provider': loadbalancer.provider,
+        }}
+        resp = {'loadbalancer': {'id': loadbalancer_id, 'provider': 'haproxy'}}
+        lbaas.create_loadbalancer.return_value = resp
+        m_driver._get_vip_port.return_value = {'id': mock.sentinel.port_id}
+
+        ret = cls._create_loadbalancer(m_driver, loadbalancer)
+        lbaas.create_loadbalancer.assert_called_once_with(req)
+        self.assertIsNone(ret)
+
     def test_find_loadbalancer(self):
         lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
         cls = d_lbaasv2.LBaaSv2Driver
@@ -268,8 +363,7 @@ class TestLBaaSv2Driver(test_base.TestCase):
             provider='haproxy', security_groups=[])
         loadbalancer_id = '00EE9E11-91C2-41CF-8FD4-7970579E5C4C'
         resp = {'loadbalancers': [{'id': loadbalancer_id,
-                                   'provider': 'haproxy',
-                                   'provisioning_status': 'ACTIVE'}]}
+                                   'provider': 'haproxy'}]}
         lbaas.list_loadbalancers.return_value = resp
         m_driver._get_vip_port.return_value = {'id': mock.sentinel.port_id}
 
@@ -283,7 +377,6 @@ class TestLBaaSv2Driver(test_base.TestCase):
             self.assertEqual(getattr(loadbalancer, attr),
                              getattr(ret, attr))
         self.assertEqual(loadbalancer_id, ret.id)
-        m_driver.release_loadbalancer.assert_not_called()
 
     def test_find_loadbalancer_not_found(self):
         lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
@@ -302,30 +395,6 @@ class TestLBaaSv2Driver(test_base.TestCase):
             vip_address=str(loadbalancer.ip),
             vip_subnet_id=loadbalancer.subnet_id)
         self.assertIsNone(ret)
-        m_driver.release_loadbalancer.assert_not_called()
-
-    def test_find_loadbalancer_error(self):
-        lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
-        cls = d_lbaasv2.LBaaSv2Driver
-        m_driver = mock.Mock(spec=d_lbaasv2.LBaaSv2Driver)
-        loadbalancer = obj_lbaas.LBaaSLoadBalancer(
-            name='TEST_NAME', project_id='TEST_PROJECT', ip='1.2.3.4',
-            subnet_id='D3FA400A-F543-4B91-9CD3-047AF0CE42D1')
-        loadbalancer_id = '00EE9E11-91C2-41CF-8FD4-7970579E5C4C'
-        resp = {'loadbalancers': [{'id': loadbalancer_id,
-                                   'provider': 'haproxy',
-                                   'provisioning_status': 'ERROR'}]}
-        lbaas.list_loadbalancers.return_value = resp
-        m_driver._get_vip_port.return_value = {'id': mock.sentinel.port_id}
-
-        ret = cls._find_loadbalancer(m_driver, loadbalancer)
-        lbaas.list_loadbalancers.assert_called_once_with(
-            name=loadbalancer.name,
-            project_id=loadbalancer.project_id,
-            vip_address=str(loadbalancer.ip),
-            vip_subnet_id=loadbalancer.subnet_id)
-        self.assertIsNone(ret)
-        m_driver.release_loadbalancer.assert_called_once()
 
     def test_create_listener(self):
         lbaas = self.useFixture(k_fix.MockLBaaSClient()).client
@@ -614,7 +683,8 @@ class TestLBaaSv2Driver(test_base.TestCase):
                                       find)
 
         m_driver._wait_for_provisioning.assert_has_calls(
-            [mock.call(loadbalancer, t) for t in timer])
+            [mock.call(loadbalancer, t, d_lbaasv2._LB_STS_POLL_FAST_INTERVAL)
+             for t in timer])
         m_driver._ensure.assert_has_calls(
             [mock.call(obj, create, find) for _ in timer])
         self.assertEqual(expected_result, ret)
@@ -635,7 +705,8 @@ class TestLBaaSv2Driver(test_base.TestCase):
                           loadbalancer, obj, create, find)
 
         m_driver._wait_for_provisioning.assert_has_calls(
-            [mock.call(loadbalancer, t) for t in timer])
+            [mock.call(loadbalancer, t, d_lbaasv2._LB_STS_POLL_FAST_INTERVAL)
+             for t in timer])
         m_driver._ensure.assert_has_calls(
             [mock.call(obj, create, find) for _ in timer])
 
