@@ -15,9 +15,10 @@
 
 import itertools
 import mock
+import uuid
+
 import os_vif.objects.network as osv_network
 import os_vif.objects.subnet as osv_subnet
-from oslo_utils import uuidutils
 
 from kuryr_kubernetes import constants as k_const
 from kuryr_kubernetes.controller.drivers import base as drv_base
@@ -351,12 +352,13 @@ class TestLBaaSSpecHandler(test_base.TestCase):
 class FakeLBaaSDriver(drv_base.LBaaSDriver):
 
     def ensure_loadbalancer(self, name, project_id, subnet_id, ip,
-                            security_groups_ids, service_type):
+                            security_groups_ids, service_type, provider=None):
         return obj_lbaas.LBaaSLoadBalancer(name=name,
                                            project_id=project_id,
                                            subnet_id=subnet_id,
                                            ip=ip,
-                                           id=uuidutils.generate_uuid())
+                                           id=str(uuid.uuid4()),
+                                           provider=provider)
 
     def ensure_listener(self, loadbalancer, protocol, port,
                         service_type='ClusterIP'):
@@ -369,7 +371,7 @@ class FakeLBaaSDriver(drv_base.LBaaSDriver):
                                        loadbalancer_id=loadbalancer.id,
                                        protocol=protocol,
                                        port=port,
-                                       id=uuidutils.generate_uuid())
+                                       id=str(uuid.uuid4()))
 
     def ensure_pool(self, loadbalancer, listener):
         return obj_lbaas.LBaaSPool(name=listener.name,
@@ -377,7 +379,7 @@ class FakeLBaaSDriver(drv_base.LBaaSDriver):
                                    loadbalancer_id=loadbalancer.id,
                                    listener_id=listener.id,
                                    protocol=listener.protocol,
-                                   id=uuidutils.generate_uuid())
+                                   id=str(uuid.uuid4()))
 
     def ensure_member(self, loadbalancer, pool, subnet_id, ip, port,
                       target_ref_namespace, target_ref_name
@@ -389,7 +391,7 @@ class FakeLBaaSDriver(drv_base.LBaaSDriver):
                                      subnet_id=subnet_id,
                                      ip=ip,
                                      port=port,
-                                     id=uuidutils.generate_uuid())
+                                     id=str(uuid.uuid4()))
 
     def release_loadbalancer(self, loadbalancer):
         pass
@@ -429,7 +431,7 @@ class FakeLBaaSDriver(drv_base.LBaaSDriver):
 
     def ensure_pool_attached_to_lb(self, loadbalancer, namespace,
                                    svc_name, protocol):
-        return obj_lbaas.LBaaSPool(id=uuidutils.generate_uuid(),
+        return obj_lbaas.LBaaSPool(id=str(uuid.uuid4()),
                                    loadbalancer_id=loadbalancer.id,
                                    project_id=loadbalancer.project_id,
                                    protocol=protocol)
@@ -445,6 +447,7 @@ class FakeLBaaSDriver(drv_base.LBaaSDriver):
 
 
 class TestLoadBalancerHandler(test_base.TestCase):
+    @mock.patch('kuryr_kubernetes.config.CONF')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.ServicePubIpDriver.get_instance')
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
@@ -454,17 +457,44 @@ class TestLoadBalancerHandler(test_base.TestCase):
     @mock.patch('kuryr_kubernetes.controller.drivers.base'
                 '.LBaaSDriver.get_instance')
     def test_init(self, m_get_drv_lbaas, m_get_drv_project,
-                  m_get_drv_subnets, m_get_drv_service_pub_ip):
+                  m_get_drv_subnets, m_get_drv_service_pub_ip, m_cfg):
         m_get_drv_lbaas.return_value = mock.sentinel.drv_lbaas
         m_get_drv_project.return_value = mock.sentinel.drv_project
         m_get_drv_subnets.return_value = mock.sentinel.drv_subnets
         m_get_drv_service_pub_ip.return_value = mock.sentinel.drv_lb_ip
+        m_cfg.kubernetes.endpoints_driver_octavia_provider = 'default'
         handler = h_lbaas.LoadBalancerHandler()
 
         self.assertEqual(mock.sentinel.drv_lbaas, handler._drv_lbaas)
         self.assertEqual(mock.sentinel.drv_project, handler._drv_pod_project)
         self.assertEqual(mock.sentinel.drv_subnets, handler._drv_pod_subnets)
         self.assertEqual(mock.sentinel.drv_lb_ip, handler._drv_service_pub_ip)
+        self.assertIsNone(handler._lb_provider)
+
+    @mock.patch('kuryr_kubernetes.config.CONF')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.ServicePubIpDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.PodSubnetsDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.PodProjectDriver.get_instance')
+    @mock.patch('kuryr_kubernetes.controller.drivers.base'
+                '.LBaaSDriver.get_instance')
+    def test_init_provider_ovn(self, m_get_drv_lbaas, m_get_drv_project,
+                               m_get_drv_subnets, m_get_drv_service_pub_ip,
+                               m_cfg):
+        m_get_drv_lbaas.return_value = mock.sentinel.drv_lbaas
+        m_get_drv_project.return_value = mock.sentinel.drv_project
+        m_get_drv_subnets.return_value = mock.sentinel.drv_subnets
+        m_get_drv_service_pub_ip.return_value = mock.sentinel.drv_lb_ip
+        m_cfg.kubernetes.endpoints_driver_octavia_provider = 'ovn'
+        handler = h_lbaas.LoadBalancerHandler()
+
+        self.assertEqual(mock.sentinel.drv_lbaas, handler._drv_lbaas)
+        self.assertEqual(mock.sentinel.drv_project, handler._drv_pod_project)
+        self.assertEqual(mock.sentinel.drv_subnets, handler._drv_pod_subnets)
+        self.assertEqual(mock.sentinel.drv_lb_ip, handler._drv_service_pub_ip)
+        self.assertEqual('ovn', handler._lb_provider)
 
     def test_on_present(self):
         lbaas_spec = mock.sentinel.lbaas_spec
@@ -793,8 +823,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
     def test_sync_lbaas_members(self, m_get_drv_lbaas, m_get_drv_project,
                                 m_get_drv_subnets):
         # REVISIT(ivc): test methods separately and verify ensure/release
-        project_id = uuidutils.generate_uuid()
-        subnet_id = uuidutils.generate_uuid()
+        project_id = str(uuid.uuid4())
+        subnet_id = str(uuid.uuid4())
         current_ip = '1.1.1.1'
         current_targets = {
             '1.1.1.101': (1001, 10001),
@@ -827,8 +857,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
     def test_sync_lbaas_members_udp(self, m_get_drv_lbaas,
                                     m_get_drv_project, m_get_drv_subnets):
         # REVISIT(ivc): test methods separately and verify ensure/release
-        project_id = uuidutils.generate_uuid()
-        subnet_id = uuidutils.generate_uuid()
+        project_id = str(uuid.uuid4())
+        subnet_id = str(uuid.uuid4())
         current_ip = '1.1.1.1'
         current_targets = {
             '1.1.1.101': (1001, 10001),
@@ -861,8 +891,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
     def test_sync_lbaas_members_svc_listener_port_edit(
             self, m_get_drv_lbaas, m_get_drv_project, m_get_drv_subnets):
         # REVISIT(ivc): test methods separately and verify ensure/release
-        project_id = uuidutils.generate_uuid()
-        subnet_id = uuidutils.generate_uuid()
+        project_id = str(uuid.uuid4())
+        subnet_id = str(uuid.uuid4())
         current_ip = '1.1.1.1'
         current_targets = {
             '1.1.1.101': (1001, 10001)}
@@ -914,8 +944,8 @@ class TestLoadBalancerHandler(test_base.TestCase):
                 '.LBaaSDriver.get_instance')
     def test_add_new_members_udp(self, m_get_drv_lbaas,
                                  m_get_drv_project, m_get_drv_subnets):
-        project_id = str(uuidutils.generate_uuid())
-        subnet_id = str(uuidutils.generate_uuid())
+        project_id = str(uuid.uuid4())
+        subnet_id = str(uuid.uuid4())
         current_ip = '1.1.1.1'
         current_targets = {
             '1.1.1.101': (1001, 10001),
