@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import os
 import random
 import socket
 import time
@@ -33,7 +32,8 @@ LOG = log.getLogger(__name__)
 
 VALID_MULTI_POD_POOLS_OPTS = {'noop': ['neutron-vif',
                                        'nested-vlan',
-                                       'nested-macvlan'],
+                                       'nested-macvlan',
+                                       'sriov'],
                               'neutron': ['neutron-vif'],
                               'nested': ['nested-vlan'],
                               }
@@ -128,10 +128,10 @@ def exponential_sleep(deadline, attempt, interval=DEFAULT_INTERVAL):
 
 
 def get_node_name():
-    try:
-        return os.environ['KUBERNETES_NODE_NAME']
-    except KeyError:
-        return socket.gethostname()
+    # leader-elector container based on K8s way of doing leader election is
+    # assuming that hostname it sees is the node id. Containers within a pod
+    # are sharing the hostname, so this will match what leader-elector returns.
+    return socket.gethostname()
 
 
 def get_leader_name():
@@ -172,3 +172,17 @@ def extract_pod_annotation(annotation):
         obj = vif.PodState(default_vif=obj)
 
     return obj
+
+
+def has_limit(quota):
+    NO_LIMIT = -1
+    return quota != NO_LIMIT
+
+
+def is_available(resource, resource_quota, neutron_func):
+    qnt_resources = len(neutron_func().get(resource))
+    availability = resource_quota - qnt_resources
+    if availability <= 0:
+        LOG.error("Quota exceeded for resource: %s", resource)
+        return False
+    return True
